@@ -11,7 +11,7 @@ import re
 import threading
 import time
 import urllib.parse
-from typing import Optional, Callable
+from typing import Optional, Callable, Type
 
 from loguru import logger
 
@@ -32,6 +32,9 @@ URL_MATCH_RULE = re.compile(
 
 
 class MetaClass(type):
+    _instances = {}
+    _lock = threading.Lock()
+
     def __new__(cls, name, bases, dct):  # noqa
         def wrapper(raw_method):
             def inner(self, *args, **kwargs):
@@ -51,6 +54,14 @@ class MetaClass(type):
         # 创建并返回新的类
         return super(MetaClass, cls).__new__(cls, name, bases, dct)
 
+    def __call__(cls, *args, **kwargs):
+        key = (cls, args, tuple(kwargs.items()))
+        with cls._lock:
+            if cls not in cls._instances:
+                instance = super().__call__(*args, **kwargs)
+                cls._instances[key] = instance
+        return cls._instances[key]
+
 
 class Proxy:
     def __init__(
@@ -148,6 +159,10 @@ class BaseProxy(metaclass=MetaClass):
 
         return proxy
 
+    @classmethod
+    def build(cls, **options):
+        return pandora.invoke(cls, kwargs=options)
+
 
 class ApiProxy(BaseProxy):
     def __init__(
@@ -193,12 +208,13 @@ class ApiProxy(BaseProxy):
     def get(self, timeout=None) -> Proxy:
         # 这个要加锁, 不然多线程会都去提取代理
         with self.lock:
-            try:
-                proxy = self.container.get(timeout=1)
-            except queue.Empty:
-                self.fetch(timeout)
-            else:
-                return Proxy(proxy)
+            while True:
+                try:
+                    proxy = self.container.get(timeout=1)
+                except queue.Empty:
+                    self.fetch(timeout)
+                else:
+                    return Proxy(proxy)
 
     def fetch(self, timeout=None):
         if timeout is None:
@@ -264,14 +280,14 @@ class RedisProxy(BaseProxy):
             timeout = math.inf
 
         start = time.time()
-        while True:
+        while time.time() - start < timeout:
             proxy = self.container.pop(self.key)
             if not proxy:
-                if time.time() - start > timeout: raise TimeoutError
                 logger.warning(f'[获取代理失败] ref: {self}')
                 time.sleep(1)
             else:
                 return Proxy(proxy)
+        raise TimeoutError
 
     def __str__(self):
         return f'<RedisProxy [key: {self.key} | options:{self.options}]>'
@@ -321,12 +337,14 @@ class Manager:
         """
         with self._context:
             for config in configs:
-                config["ref"] = pandora.load_objects(config["ref"])
+                ref = config["ref"] = pandora.load_objects(config["ref"])
+                ref: Type[BaseProxy]
+
                 rkey = self.get_rkey(config)
 
                 if not hasattr(self._local, rkey):
                     if rkey not in self.container:
-                        self.container[rkey] = pandora.invoke(config["ref"], kwargs=config)
+                        self.container[rkey] = ref.build(**config)
 
                     # 获取代理模型
                     ins: BaseProxy = self.container[rkey]
@@ -422,6 +440,5 @@ class Manager:
 manager = Manager()
 
 if __name__ == '__main__':
-    p = CustomProxy("127.0.0.1:7890")
+    p = CustomProxy("127.0.0.1:7890", scheme="socks5")
     print(p.get())
-    print(pandora.prepare(CustomProxy))
