@@ -5,16 +5,21 @@
 import ast
 import collections
 import importlib
+import importlib.metadata as importlib_metadata
 import importlib.util
 import inspect
 import json
 import linecache
 import os
 import re
+import subprocess
 import sys
-from typing import Any, List, Union
+from typing import Any, List, Union, Mapping
+
+from loguru import logger
 
 JSONP_REGEX = re.compile(r'\S+?\((?P<obj>[\s\S]*)\)')
+PACKAGE_REGEX = re.compile(r"([a-zA-Z0-9_\-]+)([<>=]*)([\d.]*)")
 
 
 def load_objects(path_or_reference, reload=False):
@@ -33,13 +38,12 @@ def load_objects(path_or_reference, reload=False):
         try:
             module_name = os.path.splitext(os.path.basename(path_or_reference))[0]
             spec = importlib.util.spec_from_file_location(module_name, path_or_reference)
-            if spec and spec.loader:
-                module = importlib.util.module_from_spec(spec)
-                sys.modules[module_name] = module
-                spec.loader.exec_module(module)
-                return module
-            else:
-                raise ImportError(f"无法从文件路径导入模块：{path_or_reference}")
+
+            assert spec and spec.loader, ImportError(f"无法从文件路径导入模块：{path_or_reference}")
+            module = importlib.util.module_from_spec(spec)
+            sys.modules[module_name] = module
+            spec.loader.exec_module(module)
+            return module
         except Exception as e:
             raise e
     else:
@@ -64,6 +68,36 @@ def load_objects(path_or_reference, reload=False):
         raise ImportError(f"无法导入指定路径或引用: {path_or_reference}")
 
 
+def require(package_spec: str) -> str:
+    """
+    依赖 python 包
+
+    :param package_spec: pymongo==4.6.0 / pymongo
+    :return: 安装的包版本号
+    """
+    # 分离包名和版本规范
+    match = PACKAGE_REGEX.match(package_spec)
+    assert match, ValueError("无效的包规范")
+
+    package, operator, required_version = match.groups()
+
+    try:
+        # 获取已安装的包版本
+        installed_version = importlib_metadata.version(package)
+        # 检查是否需要安装或更新
+        if required_version and not eval(f'{installed_version!r} {operator} {required_version!r}'):
+            raise importlib_metadata.PackageNotFoundError
+        else:
+            return installed_version
+
+    except importlib_metadata.PackageNotFoundError:
+        # 包没有安装或版本不符合要求
+        install_command = package_spec if required_version else package
+        subprocess.check_call([sys.executable, "-m", "pip", "install", install_command])
+        logger.debug(f"'{install_command}'已安装/更新。")
+        return importlib_metadata.version(package)
+
+
 def invoke(func, args=None, kwargs: dict = None, annotations: dict = None, namespace: dict = None):
     """
     调用函数, 自动修正参数
@@ -80,7 +114,7 @@ def invoke(func, args=None, kwargs: dict = None, annotations: dict = None, names
 
 
 def prepare(func, args=None, kwargs: dict = None, annotations: dict = None, namespace: dict = None):
-    assert callable(func), f"func must be callable, but got {type(func)}"
+    assert callable(func), ValueError(f"func must be callable, but got {type(func)}")
     prepared = collections.namedtuple("prepared", ["func", "args", "kwargs"])
 
     args = args or []
@@ -93,7 +127,7 @@ def prepare(func, args=None, kwargs: dict = None, annotations: dict = None, name
 
     try:
         parameters = inspect.signature(func).parameters
-    except:
+    except:  # noqa
         parameters = {}
         new_args = [*args]
         kwargs and new_args.append(kwargs)
@@ -121,7 +155,7 @@ def prepare(func, args=None, kwargs: dict = None, annotations: dict = None, name
             value = namespace[param.name]
 
         # 参数类型存在于 annotations, 并且还可以从 args 里面取值, 并且刚好取到的对应的值也是当前类型 -> 直接从 args 里面取
-        elif param.annotation in annotations and index < len(args) and type(args[index]) == param.annotation:
+        elif param.annotation in annotations and index < len(args) and type(args[index]) is param.annotation:
             value = args[index]
             index += 1
 
@@ -149,14 +183,12 @@ def prepare(func, args=None, kwargs: dict = None, annotations: dict = None, name
     return prepared(func=func, args=new_args, kwargs=new_kwargs)
 
 
-def ensure_type(_object, _type):
-    if not isinstance(_object, _type):
-        return _type(_object)
-    else:
-        return _object
-
-
-def iterable(_object: Any, enforce=(dict, str, bytes), exclude=(), convert_null=True) -> List[Any]:
+def iterable(
+        _object: Any,
+        enforce=(dict, str, bytes, collections.UserDict, Mapping),
+        exclude=(),
+        convert_null=True
+) -> List[Any]:
     """
     用列表将 `exclude` 类型中的其他类型包装起来
 
@@ -221,14 +253,12 @@ def json_or_eval(text, jsonp=False, errors="strict", _step=0, **kwargs) -> Union
     for func in funcs:
         try:
             return func()
-        except:
+        except:  # noqa
             pass
 
     else:
-        if errors != 'ignore':
-            raise ValueError(f'illegal json string: `{text}`')
-        else:
-            return text
+        assert errors == "ignore", ValueError(f'illegal json string: `{text}`')
+        return text
 
 
 def get_simple_stack(e):
@@ -256,9 +286,76 @@ def get_simple_stack(e):
     return formatted_trace
 
 
+def clean_rows(*rows: dict, **layout):
+    """
+    清洗数据
+
+    :param rows: 需要处理的数据
+    :param layout:
+        - rename: 修改名字
+        - show: 移除 / 保留 / 函数
+        - factory: 工厂函数, 动态处理
+        - default: 默认值
+    :return:
+    """
+
+    def _rename(rule: dict, data: dict):
+        for oldname, newname in rule.items():
+            data[newname] = data.pop(oldname, None)
+
+    def _show(rule: dict, data: dict):
+        for key, flag in rule.items():
+            if (
+                    callable(flag) and not invoke(flag, args=[data.get(key, None)], kwargs={"row": row})
+                    or not flag
+            ):
+                rule.pop(key, None)
+
+    def _factory(rule: dict, data: dict):
+        for key, func in rule.items():
+            if isinstance(func, str):
+                func = load_objects(func)
+            data[key] = invoke(func, args=[data.get(key, None)], kwargs={"row": row})
+
+    def _default(rule: dict, data: dict):
+        for key, default in rule.items():
+            data.setdefault(key, default)
+
+    flows = [
+        (layout.get("default") or {}, _default),
+        (layout.get("show") or {}, _show),
+        (layout.get("factory") or {}, _factory),
+        (layout.get("rename") or {}, _rename),
+
+    ]
+    for row in rows:
+        for _rule, flow in flows:
+            _rule and flow(_rule, row)
+
+    else:
+        return list(rows)
+
+
+class Method:
+    def __init__(self, func):
+        self.func = func
+
+    def __get__(self, obj, objtype=None):
+        if obj is None:
+            # 当作为类方法调用
+            return lambda *args, **kwargs: self.func(objtype, *args, **kwargs)
+        else:
+            # 当作为实例方法调用
+            return lambda *args, **kwargs: self.func(obj, *args, **kwargs)
+
+
 if __name__ == '__main__':
-    def fun(a: int, *args, c: int = 1, **kwargs):
-        print(a, args, c, kwargs)
-
-
-    print(invoke(fun, args=[1], annotations={int: 999}))
+    # print(require("pandas"))
+    my_data = [{"id": i, "name": i} for i in range(100)]
+    print(clean_rows(
+        *my_data,
+        default={"hobby": "nothing"},
+        rename={"id": "uid"},
+        factory={"name": lambda x: "name: " + str(x)}
+    ))
+    print(my_data)
