@@ -1,10 +1,13 @@
 from loguru import logger
 
+import bricks
 from bricks import Request, const
 from bricks.core import signals
 from bricks.spider import air
 from bricks.spider.air import Context
 
+bricks.T.set("name", "kem")
+
 
 class MySpider(air.Spider):
 
@@ -14,7 +17,7 @@ class MySpider(air.Spider):
         # 如果有多个, 你有两种方案
         # 1. 将种子全部放置在一个列表里面, yield 出去, 如 return [{"page":1}, {"page":2}, {"page":3}]
         # 2. 使用生成器, 每次生产一部分, 如 yield {"page":1}, yield {"page":2}, yield {"page":3}
-        return {"page": 1}
+        return [{"page":1}, {"page":2}, {"page":3}]
 
     def make_request(self, context: Context) -> Request:
         # 之前定义的种子会被投放至任务队列, 之后会被取出来, 迁入至 context 对象内
@@ -55,7 +58,8 @@ class MySpider(air.Spider):
         context.success()
 
     @staticmethod
-    def turn_page(context: Context):
+    def turn_page():
+        context: Context = Context.get_context()
         # 判断是否存在下一页
         has_next = context.response.get('data.hasNextPage')
         if has_next == 1:
@@ -83,20 +87,21 @@ class MySpider(air.Spider):
 
 if __name__ == '__main__':
     spider = MySpider(
+        concurrency=3
         # # 设置代理模式 1, 该模式适用于: 你已经将代理提取至 Redis 的 proxy 里面
         # # 这样设置的话就会自动去取
-        proxy={
-            "ref": "bricks.lib.proxies.RedisProxy",  # 指向 Redis
-            "key": "proxy",  # 指向代理 Key
-            # 这个不写默认指向本地 Redis, 无密码的
-            "options": {
-                "host": "127.0.0.1",
-                "port": 6379,
-                # "password": "xsxsxax"
-            },
-            "threshold": 100,  # 一个代理最多使用多少次, 到这个次数之后就会归还到Redis, 然后重新拿, 默认不归还
-            "scheme": "socks5"  # 代理协议, 默认是 http
-        }
+        # proxy={
+        #     "ref": "bricks.lib.proxies.RedisProxy",  # 指向 Redis
+        #     "key": "proxy",  # 指向代理 Key
+        #     # 这个不写默认指向本地 Redis, 无密码的
+        #     "options": {
+        #         "host": "127.0.0.1",
+        #         "port": 6379,
+        #         # "password": "xsxsxax"
+        #     },
+        #     "threshold": 100,  # 一个代理最多使用多少次, 到这个次数之后就会归还到Redis, 然后重新拿, 默认不归还
+        #     "scheme": "socks5"  # 代理协议, 默认是 http
+        # },
 
         # # 设置代理模式 2, 该模式适用于: 指向固定代理, 如 http://127.0.0.1:7890
         # # 这样设置的话就会自动去取
@@ -114,6 +119,7 @@ if __name__ == '__main__':
         #     "key": "http://你的提取代理的网址",  # 指向代理 Key
         #     "threshold": 100,  # 一个代理最多使用多少次, 到这个次数之后就会归还到Redis, 然后重新拿, 默认不归还
         #     "scheme": "http"  # 代理协议, 默认是 http
-        # }
+        # },
+        # task_queue=RedisQueue()
     )
     spider.run()
